/*
 * Copyright (C) 2012  Radek Novacek <rnovacek@redhat.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include "activeconnection_nm.h"

#include <string.h>

#include "activeconnection.h"
#include "connection.h"
#include "dbus_wrapper.h"
#include "port.h"

#include "network_nm.h"

#include "activeconnection_private.h"
#include "network_private.h"

typedef struct ActiveConnectionPriv {
    DBusGProxy *proxy;
    char *objectpath;
    GHashTable *properties;
} ActiveConnectionPriv;

ActiveConnection *active_connection_from_objectpath(Network *network, const char *objectpath)
{
    ActiveConnectionPriv *priv = malloc(sizeof(ActiveConnection));
    ActiveConnection *activeConnection = active_connection_new();
    activeConnection->ports = ports_new(0);
    activeConnection->priv = priv;
    priv->objectpath = strdup(objectpath);
    priv->proxy = dbus_g_proxy_new_for_name(network_priv_get_dbus_connection(network), NM_SERVICE_DBUS, objectpath, NM_DBUS_INTERFACE_ACTIVE_CONNECTION);
    priv->properties = dbus_get_properties(priv->proxy, objectpath, NM_DBUS_INTERFACE_ACTIVE_CONNECTION);

    if (priv->properties == NULL) {
        error("Unable to get properties for object %s", objectpath);
        return NULL;
    }
    GValue *v = g_hash_table_lookup(priv->properties, "Devices");
    warn("ActiveConnection from object_path: %s", G_VALUE_TYPE_NAME(v));
    GPtrArray *array = dbus_property_array(priv->properties, "Devices");
    char *devicepath;
    Port *port;
    for (guint i = 0; i < array->len; ++i) {
        devicepath = (char *) g_ptr_array_index(array, i);
        port = ports_find_by_uuid(network->ports, devicepath);
        if (port == NULL) {
            warn("No such port: %s", devicepath);
            continue;
        }
        ports_add(activeConnection->ports, port);
    }
    const char *connectionpath = dbus_property_objectpath(priv->properties, "Connection");
    Connection *connection = connections_find_by_uuid(network->connections, connectionpath);
    if (connection != NULL) {
        activeConnection->connection = connection;
    } else {
        warn("No such connection: %s", connectionpath);
    }
    return activeConnection;
}

void activeconnection_priv_free(void *priv)
{
    ActiveConnectionPriv *p = priv;
    free(p->objectpath);
    g_hash_table_destroy(p->properties);
    g_object_unref(p->proxy);
    free(p);
}
